/* integrands used in quadrature integration with scipy's LowLevelCallables */
#include <math.h>

#define KB 8.617333262145179e-05

/* FFHS functions */
double ffhsSD(double x, void *user_data) {
	double *c = (double *)user_data;

	double omega = c[0];
	double tau = c[1];

	double u = x*x;
	double res = u*u * tau;

	res /= 81. + 9.*u - 2.*u*u + u*u*u;
	res /= u*u + omega*omega * tau*tau;
	return res;
}

/* log-gaussian functions */
double logNormalDist(double tau, double tau0, double sigma) {
    return exp(- pow((log(tau/tau0) / sigma), 2) / 2.) / sqrt(2*M_PI)/sigma;
}

double logGaussianSD_high(double u, void *user_data) {
	double *c = (double *)user_data;

	double omega = c[0];
	double tau = c[1];
	double sigma = c[2];

    double uu = exp(-u);
    double dist = logNormalDist(1./uu, tau, sigma);

    return dist * omega * uu / (pow(uu, 2) + pow(omega, 2));
}

double logGaussianSD_low(double u, void *user_data) {
	double *c = (double *)user_data;

	double omega = c[0];
	double tau = c[1];
	double sigma = c[2];

    double uu = exp(u);

    double dist = logNormalDist(uu, tau, sigma);

    return dist * omega * uu / (1. + pow(omega*uu, 2));
}

double logGaussianCorrelation(double x, void *user_data) {
    double *c = (double *)user_data;

    double t = c[0];
	double tau = c[1];
	double sigma = c[2];

    double uu = exp(x);

    double dist = logNormalDist(uu, tau, sigma);

    return dist * exp(-t/uu);
}


double normalDist(double x, double x0, double sigma) {
    return exp(- pow((x-x0) / sigma, 2) / 2.) / sqrt(2 * M_PI) / sigma;
}

double rate(double tau0, double ea, double t) {
    return exp(-ea / t / KB) / tau0;
}

double energyDist_SD(double x, void *user_data) {
    double *c = (double *)user_data;

    double omega = c[0];
    double tau0 = c[1];
    double e_m = c[2];
    double e_b = c[3];
    double temp = c[4];

    double r = rate(tau0, x, temp);

    return r/(pow(r, 2) + pow(omega, 2)) * normalDist(x, e_m, e_b);
}
